"""Google news model."""

# pylint: disable=broad-exception-caught
import datetime

import pytest_is_running
import requests_cache
from bs4 import BeautifulSoup
from dateutil import parser

from ...cache import MEMORY
from ...vendor.pygooglenews import GoogleNews  # type: ignore
from ..league import League, long_name
from ..news_model import NewsModel


def _create_google_news_models(
    query: str,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    league: League,
) -> list[NewsModel]:
    gn = GoogleNews(session=session)
    to_dt = dt - datetime.timedelta(days=1)
    from_dt = to_dt - datetime.timedelta(days=1)
    if from_dt.date() >= datetime.datetime.now().date():
        from_dt = datetime.datetime.now() - datetime.timedelta(days=1)
    search_query = f'"{query}" + (sport OR {league} OR "{long_name(league)}")'
    try:
        s = gn.search(query=search_query, from_=str(from_dt), to_=str(to_dt))
        return sorted(
            [
                NewsModel(
                    title=x["title"],
                    published=parser.parse(x["published"]),
                    summary=BeautifulSoup(x["summary"], features="lxml").get_text(),
                    source=x["source"]["title"],
                )
                for x in s["entries"]
            ],
            key=lambda x: x.published,
        )
    except Exception:
        return []


@MEMORY.cache(ignore=["session"])
def _cached_create_google_news_models(
    query: str,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    league: League,
) -> list[NewsModel]:
    return _create_google_news_models(query, session, dt, league)


def create_google_news_models(
    query: str,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    league: League,
) -> list[NewsModel]:
    """Create news models from google."""
    if dt > datetime.datetime.now().replace(tzinfo=dt.tzinfo) + datetime.timedelta(
        days=1
    ):
        return []
    if not pytest_is_running.is_running() and dt < datetime.datetime.now().replace(
        tzinfo=dt.tzinfo
    ) - datetime.timedelta(days=3):
        return _cached_create_google_news_models(query, session, dt, league)
    with session.cache_disabled():
        return _create_google_news_models(query, session, dt, league)



"""The enumeration of the different supported positions."""

from enum import StrEnum


class Position(StrEnum):
    """An enumeration over the different positions."""

    DEFENSEMAN = "D"
    CENTRE = "C"
    GOALTENDER = "G"
    LEFT_WING = "LW"
    RIGHT_WING = "RW"
    FORWARD = "F"
    UNASSIGNED = "UN"



"""LaLiga ESPN league model."""

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...epl.position import Position
from ...espn.espn_league_model import ESPNLeagueModel
from ...league import League

_SEASON_URL = (
    "http://sports.core.api.espn.com/v2/sports/soccer/leagues/esp.1/seasons?limit=100"
)


class LaLigaESPNLeagueModel(ESPNLeagueModel):
    """LaLiga ESPN implementation of the league model."""

    def __init__(self, session: ScrapeSession, position: int | None = None) -> None:
        super().__init__(_SEASON_URL, League.LALIGA, session, position=position)

    @classmethod
    def name(cls) -> str:
        return "laliga-espn-league-model"

    @classmethod
    def position_validator(cls) -> dict[str, str]:
        positions = {str(x): str(x) for x in Position}
        positions["RCF"] = str(Position.CENTRE_FORWARD_RIGHT)
        return positions



"""NCAAB OddsPortal league model."""

# pylint: disable=line-too-long

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...league import League
from ...oddsportal.oddsportal_league_model import OddsPortalLeagueModel


class NCAABOddsPortalLeagueModel(OddsPortalLeagueModel):
    """NCAAB OddsPortal implementation of the league model."""

    def __init__(self, session: ScrapeSession, position: int | None = None) -> None:
        super().__init__(League.NCAAB, session, position=position)

    @classmethod
    def name(cls) -> str:
        return "ncaab-oddsportal-league-model"



"""Footballdata umpire model."""

# pylint: disable=too-many-arguments
from ...cache import MEMORY
from ..umpire_model import UmpireModel


@MEMORY.cache
def create_footballdata_umpire_model(
    name: str,
    version: str,
) -> UmpireModel:
    """Create a umpire model based off footballdata."""
    return UmpireModel(
        identifier=name,
        name=name,
        birth_date=None,
        age=None,
        birth_address=None,
        high_school=None,
        version=version,
    )
